// emacs: this is -*- c++ -*-
//
//   @file    ploughshare.h        
//                    
//
//   $Id: ploughshare.h, v0.0   Sun  6 May 2018 00:39:50 CEST sutt $
//
//   Copyright (C) 2018 M Sutton (sutt@cern.ch)    


#ifndef  PLOUGHSHARE_H
#define  PLOUGHSHARE_H

#include <iostream>
#include <string>
#include <vector>
#include <set>
#include <map>


class ploughshare {

public:

  /// useful utility class to store information on the dataset name, 
  /// the full path to the directory, and the full paths of all the 
  /// actual grids themselves

  class dataset_t { 

  public:

    dataset_t() { }
    dataset_t( const std::string& n, const std::string& p ) : mname(n), mpath(p) { std::cout << "dataset" << n << std::endl; } 
    dataset_t( const std::string& n, const std::string& p, const std::set<std::string>& s ) : mname(n), mpath(p), mgrids(s) { } 
    
    std::string                  name()  const { return mname; }
    std::string                  path()  const { return mpath; }
    size_t                       size()  const { return mgrids.size(); }
    const std::set<std::string>& grids() const { return mgrids; }
    

  private:
    std::string           mname;
    std::string           mpath;
    std::set<std::string> mgrids;
  };


public:

  ploughshare() : mverbose(false) { init(); } 

  virtual ~ploughshare() { } 

  /// fetch a grid (from the web if need be) - return how many grids in dataset 

  size_t fetch( const std::string& dataset );

  /// enable verbose printing during fetching

  bool verbose( bool b ) { return mverbose=b; }


  /// return the standard search path for finding the grids
  const std::vector<std::string> path() const { return mpath; }

  /// return the local paths for all available grids in the specified dataset

  const std::set<std::string> find( const std::string& dataset ) const;


  /// return the full database entry for the specified dataset

  const dataset_t& dataset( const std::string& dataset ) const { 
    std::map<std::string, dataset_t >::const_iterator itr = mmap.find( dataset );
    if ( itr!=mmap.end() ) return itr->second;
    else {
      std::cerr << "cannot find " << dataset << std::endl;
      throw std::exception();
    }
  }


  /// return the full path to a specified dataset

  const std::string path( const std::string& dataset ) const { 
    std::map<std::string, dataset_t >::const_iterator itr = mmap.find( dataset );
    if ( itr!=mmap.end() ) return itr->second.path();
    else                   return "";
  }


  /// return a vector containing the strings with the full path
  /// to the grids in the dataset

  const std::vector<std::string> grids( const std::string& dataset ) const {
    std::vector<std::string> g;
    std::map<std::string, dataset_t >::const_iterator itr = mmap.find( dataset );
    if ( itr!=mmap.end() ) { 
      const std::set<std::string>& s = itr->second.grids();
      g.reserve(s.size());
      for ( std::set<std::string>::const_iterator it=s.begin() ; it!=s.end() ; it++ ) g.push_back( *it );
    }
    return g;
  } 


  /// how many grids are there in this dataset 

  size_t size( const std::string& dataset ) const {
    std::map<std::string, dataset_t >::const_iterator itr = mmap.find( dataset );
    if ( itr==mmap.end() ) return 0;
    return itr->second.grids().size();
  } 

protected:

  int init();

  /// actually download a grid from a url
  int download( const std::string& url );

private:

  /// overall search path 
  std::vector<std::string>           mpath;

  /// list of grids
  std::set<std::string>              mgrids;

  /// full lookup
  std::map<std::string, dataset_t >  mmap;

  bool                               mverbose;

};



inline std::ostream& operator<<( std::ostream& s, const ploughshare::dataset_t& _d ) { 
  return s << "dataset: " << _d.name(); //  << "\n" << _d.grids();
}

inline std::ostream& operator<<( std::ostream& s, const ploughshare& _p ) { 
  return s;
}


#endif  // PLOUGHSHARE_H 










